@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,10 +22,17 @@ statement, 'if a: ... else: ...' and there is no 'elif', because that is
 expressed via nesting of conditional statements.
 """
 
-from nuitka.nodes.BuiltinTypeNodes import ExpressionBuiltinBool
-from nuitka.optimizations.TraceCollections import ConstraintCollectionBranch
+from nuitka.optimizations.TraceCollections import TraceCollectionBranch
 
-from .NodeBases import ExpressionChildrenHavingBase, StatementChildrenHavingBase
+from .BuiltinTypeNodes import ExpressionBuiltinBool
+from .Checkers import checkStatementsSequenceOrNone
+from .ExpressionBases import ExpressionChildrenHavingBase
+from .NodeBases import StatementChildrenHavingBase
+from .NodeMakingHelpers import (
+    makeStatementExpressionOnlyReplacementNode,
+    wrapExpressionWithNodeSideEffects,
+    wrapStatementWithSideEffects
+)
 
 
 class ExpressionConditional(ExpressionChildrenHavingBase):
@@ -37,17 +44,19 @@ class ExpressionConditional(ExpressionChildrenHavingBase):
         "expression_no"
     )
 
-    def __init__(self, condition, yes_expression, no_expression, source_ref):
+    def __init__(self, condition, expression_yes, expression_no, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
                 "condition"      : condition,
-                "expression_yes" : yes_expression,
-                "expression_no"  : no_expression
+                "expression_yes" : expression_yes,
+                "expression_no"  : expression_no
             },
             source_ref = source_ref
         )
 
+        self.merge_traces = None
+
     def getBranches(self):
         return (
             self.getExpressionYes(),
@@ -65,26 +74,32 @@ class ExpressionConditional(ExpressionChildrenHavingBase):
         "condition"
     )
 
-    def computeExpressionRaw(self, constraint_collection):
+    def computeExpressionRaw(self, trace_collection):
         # Query the truth value after the expression is evaluated, once it is
         # evaluated in onExpression, it is known.
-        constraint_collection.onExpression(
+        trace_collection.onExpression(
             expression = self.getCondition()
         )
         condition = self.getCondition()
 
+        condition_may_raise = condition.mayRaiseException(BaseException)
+
+        if condition_may_raise:
+            trace_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
         # No need to look any further, if the condition raises, the branches do
         # not matter at all.
         if condition.willRaiseException(BaseException):
             return condition, "new_raise", """\
-Conditional statements already raises implicitly in condition, removing \
+Conditional expression already raises implicitly in condition, removing \
 branches."""
 
-        # If the condition raises, we let that escape instead, and the
-        # branches don't matter at all.
-        if condition.willRaiseException(BaseException):
-            return condition, "new_raise", """\
-Conditional expression raises in condition."""
+        if not condition_may_raise and condition.mayRaiseExceptionBool(BaseException):
+            trace_collection.onExceptionRaiseExit(
+                BaseException
+            )
 
         # Decide this based on truth value of condition.
         truth_value = condition.getTruthValue()
@@ -96,8 +111,9 @@ Conditional expression raises in condition."""
         # Continue to execute for yes branch unless we know it's not going to be
         # relevant.
         if truth_value is not False:
-            branch_yes_collection = ConstraintCollectionBranch(
-                parent = constraint_collection,
+            branch_yes_collection = TraceCollectionBranch(
+                parent = trace_collection,
+                name   = "conditional expression yes branch"
             )
 
             branch_yes_collection.computeBranch(
@@ -117,8 +133,9 @@ Conditional expression raises in condition."""
 
         # Continue to execute for yes branch.
         if truth_value is not True:
-            branch_no_collection = ConstraintCollectionBranch(
-                parent = constraint_collection,
+            branch_no_collection = TraceCollectionBranch(
+                parent = trace_collection,
+                name   = "conditional expression no branch"
             )
 
             branch_no_collection.computeBranch(
@@ -135,14 +152,12 @@ Conditional expression raises in condition."""
             branch_no_collection = None
 
         # Merge into parent execution.
-        constraint_collection.mergeBranches(
+        self.merge_traces = trace_collection.mergeBranches(
             branch_yes_collection,
             branch_no_collection
         )
 
         if truth_value is True:
-            from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects
-
             return (
                 wrapExpressionWithNodeSideEffects(
                     new_node = self.getExpressionYes(),
@@ -152,8 +167,6 @@ Conditional expression raises in condition."""
                 "Conditional expression predicted to yes case"
             )
         elif truth_value is False:
-            from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects
-
             return (
                 wrapExpressionWithNodeSideEffects(
                     new_node = self.getExpressionNo(),
@@ -203,18 +216,191 @@ Conditional expression raises in condition."""
         return False
 
     def mayRaiseExceptionBool(self, exception_type):
-        if self.getCondition().mayRaiseExceptionBool():
+        if self.getCondition().mayRaiseExceptionBool(exception_type):
             return True
 
-        if self.getExpressionYes().mayRaiseExceptionBool():
+        if self.getExpressionYes().mayRaiseExceptionBool(exception_type):
             return True
 
-        if self.getExpressionNo().mayRaiseExceptionBool():
+        if self.getExpressionNo().mayRaiseExceptionBool(exception_type):
             return True
 
         return False
 
 
+class ExpressionConditionalBoolBase(ExpressionChildrenHavingBase):
+    named_children = (
+        "left",
+        "right"
+    )
+
+    def __init__(self, left, right, source_ref):
+        ExpressionChildrenHavingBase.__init__(
+            self,
+            values     = {
+                "left"  : left,
+                "right" : right,
+            },
+            source_ref = source_ref
+        )
+
+        self.merge_traces = None
+
+    getLeft = ExpressionChildrenHavingBase.childGetter(
+        "left"
+    )
+    getRight = ExpressionChildrenHavingBase.childGetter(
+        "right"
+    )
+
+    def computeExpressionRaw(self, trace_collection):
+        # Query the truth value after the expression is evaluated, once it is
+        # evaluated in onExpression, it is known.
+        trace_collection.onExpression(
+            expression = self.getLeft()
+        )
+        left = self.getLeft()
+
+        left_may_raise = left.mayRaiseException(BaseException)
+
+        if left_may_raise:
+            trace_collection.onExceptionRaiseExit(
+                BaseException
+            )
+        # No need to look any further, if the condition raises, the branches do
+        # not matter at all.
+        if left.willRaiseException(BaseException):
+            return left, "new_raise", """\
+Conditional %s statements already raises implicitly in condition, removing \
+branches.""" % self.conditional_kind
+
+        if not left_may_raise and left.mayRaiseExceptionBool(BaseException):
+            trace_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
+        # Decide this based on truth value of condition.
+        truth_value = left.getTruthValue()
+
+        truth_value_use_left = self.conditional_kind == "or"
+        truth_value_use_right = not truth_value_use_left
+
+        right = self.getRight()
+
+        # Continue to execute for yes branch unless we know it's not going to be
+        # relevant.
+        if truth_value is not truth_value_use_left:
+            # TODO: We now know that left evaluates and we should tell the
+            # branch that.
+            branch_yes_collection = TraceCollectionBranch(
+                parent = trace_collection,
+                name   = "boolean %s right branch" % self.conditional_kind
+            )
+
+            branch_yes_collection.computeBranch(
+                branch = right
+            )
+
+            # May have just gone away, so fetch it again.
+            right = self.getRight()
+
+            # If it's aborting, it doesn't contribute to merging.
+            if right.willRaiseException(BaseException):
+                branch_yes_collection = None
+        else:
+            branch_yes_collection = None
+
+
+        if branch_yes_collection:
+            # Merge into parent execution.
+            self.merge_traces = trace_collection.mergeBranches(
+                branch_yes_collection,
+                None
+            )
+        else:
+            self.merge_traces = None
+
+        if truth_value is truth_value_use_left:
+            return (
+                left,
+                "new_expression",
+                "Conditional '%s' expression predicted to left value." % self.conditional_kind
+            )
+        elif truth_value is truth_value_use_right:
+            return (
+                wrapExpressionWithNodeSideEffects(
+                    new_node = right,
+                    old_node = left
+                ),
+                "new_expression",
+                "Conditional '%s' expression predicted right value." % self.conditional_kind
+            )
+        else:
+            return self, None, None
+
+    def mayRaiseException(self, exception_type):
+        left = self.getLeft()
+
+        if left.mayRaiseException(exception_type):
+            return True
+
+        if left.mayRaiseExceptionBool(exception_type):
+            return True
+
+        right = self.getRight()
+
+        if right.mayRaiseException(exception_type):
+            return True
+
+        return False
+
+    def mayRaiseExceptionBool(self, exception_type):
+        if self.getLeft().mayRaiseExceptionBool(exception_type):
+            return True
+
+        if self.getRight().mayRaiseExceptionBool(exception_type):
+            return True
+
+        return False
+
+    def mayHaveSideEffectsBool(self):
+        if self.getLeft().mayHaveSideEffectsBool():
+            return True
+
+        if self.getRight().mayHaveSideEffectsBool():
+            return True
+
+        return False
+
+
+class ExpressionConditionalOR(ExpressionConditionalBoolBase):
+    kind = "EXPRESSION_CONDITIONAL_OR"
+
+    def __init__(self, left, right, source_ref):
+        ExpressionConditionalBoolBase.__init__(
+            self,
+            left       = left,
+            right      = right,
+            source_ref = source_ref
+        )
+
+        self.conditional_kind = "or"
+
+
+class ExpressionConditionalAND(ExpressionConditionalBoolBase):
+    kind = "EXPRESSION_CONDITIONAL_AND"
+
+    def __init__(self, left, right, source_ref):
+        ExpressionConditionalBoolBase.__init__(
+            self,
+            left       = left,
+            right      = right,
+            source_ref = source_ref
+        )
+
+        self.conditional_kind = "and"
+
+
 class StatementConditional(StatementChildrenHavingBase):
     kind = "STATEMENT_CONDITIONAL"
 
@@ -224,6 +410,11 @@ class StatementConditional(StatementChildrenHavingBase):
         "no_branch"
     )
 
+    checkers = {
+        "yes_branch" : checkStatementsSequenceOrNone,
+        "no_branch" : checkStatementsSequenceOrNone,
+    }
+
     def __init__(self, condition, yes_branch, no_branch, source_ref):
         StatementChildrenHavingBase.__init__(
             self,
@@ -235,6 +426,8 @@ class StatementConditional(StatementChildrenHavingBase):
             source_ref = source_ref
         )
 
+        self.merge_traces = None
+
     getCondition = StatementChildrenHavingBase.childGetter("condition")
     getBranchYes = StatementChildrenHavingBase.childGetter("yes_branch")
     setBranchYes = StatementChildrenHavingBase.childSetter("yes_branch")
@@ -307,21 +500,24 @@ class StatementConditional(StatementChildrenHavingBase):
 
         return False
 
-    def computeStatement(self, constraint_collection):
-        # This is rather complex stuff, pylint: disable=R0912
+    def computeStatement(self, trace_collection):
+        # This is rather complex stuff, pylint: disable=too-many-branches,too-many-statements
 
-        # Query the truth value after the expression is evaluated, once it is
-        # evaluated in onExpression, it is known.
-        constraint_collection.onExpression(
+        trace_collection.onExpression(
             expression = self.getCondition()
         )
         condition = self.getCondition()
 
+        condition_may_raise = condition.mayRaiseException(BaseException)
+
+        if condition_may_raise:
+            trace_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
         # No need to look any further, if the condition raises, the branches do
         # not matter at all.
         if condition.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementExpressionOnlyReplacementNode
-
             result = makeStatementExpressionOnlyReplacementNode(
                 expression = condition,
                 node       = self
@@ -331,24 +527,56 @@ class StatementConditional(StatementChildrenHavingBase):
 Conditional statements already raises implicitly in condition, removing \
 branches."""
 
-        # Consider to not execute branches that we know to be true, but execute
-        # the ones that may be true, potentially both.
+        if not condition_may_raise and condition.mayRaiseExceptionBool(BaseException):
+            trace_collection.onExceptionRaiseExit(
+                BaseException
+            )
+
+
+        # Query the truth value after the expression is evaluated, once it is
+        # evaluated in onExpression, it is known.
         truth_value = condition.getTruthValue()
 
         # TODO: We now know that condition evaluates to true for the yes branch
         # and to not true for no branch, the branch collection should know that.
         yes_branch = self.getBranchYes()
+        no_branch = self.getBranchNo()
 
         # Handle branches that became empty behind our back.
         if yes_branch is not None:
             if not yes_branch.getStatements():
                 yes_branch = None
+        if no_branch is not None:
+            if not no_branch.getStatements():
+                no_branch = None
+
+        # Consider to not remove branches that we know won't be taken.
+        if yes_branch is not None and truth_value is False:
+            trace_collection.signalChange(
+                tags       = "new_statements",
+                source_ref = yes_branch.source_ref,
+                message    = "Removed conditional branch not taken due to false condition value."
+            )
+
+            self.setBranchYes(None)
+            yes_branch = None
+
+        if no_branch is not None and truth_value is True:
+            trace_collection.signalChange(
+                tags       = "new_statements",
+                source_ref = no_branch.source_ref,
+                message    = "Removed 'else' branch not taken due to true condition value."
+            )
+
+            self.setBranchNo(None)
+            no_branch = None
 
         # Continue to execute for yes branch unless we know it's not going to be
         # relevant.
-        if yes_branch is not None and truth_value is not False:
-            branch_yes_collection = ConstraintCollectionBranch(
-                parent = constraint_collection,
+        if yes_branch is not None:
+            branch_yes_collection = TraceCollectionBranch(
+                parent = trace_collection,
+                name   = "conditional yes branch",
             )
 
             branch_yes_collection.computeBranch(
@@ -364,24 +592,17 @@ branches."""
         else:
             branch_yes_collection = None
 
-        no_branch = self.getBranchNo()
-
-        # Handle branches that became empty behind our back
-        if no_branch is not None:
-            if not no_branch.getStatements():
-                no_branch = None
-
         # Continue to execute for yes branch.
-        if no_branch is not None and truth_value is not True:
-            branch_no_collection = ConstraintCollectionBranch(
-                parent = constraint_collection,
+        if no_branch is not None:
+            branch_no_collection = TraceCollectionBranch(
+                parent = trace_collection,
+                name   = "conditional no branch"
             )
 
             branch_no_collection.computeBranch(
                 branch = no_branch
             )
 
-
             # May have just gone away, so fetch it again.
             no_branch = self.getBranchNo()
 
@@ -392,32 +613,59 @@ branches."""
             branch_no_collection = None
 
         # Merge into parent execution.
-        constraint_collection.mergeBranches(
+        self.merge_traces = trace_collection.mergeBranches(
             branch_yes_collection,
             branch_no_collection
         )
 
-        # Both branches may have become empty.
+        # Both branches may have become empty, which case, the statement needs
+        # not remain.
         if yes_branch is None and no_branch is None:
-            from .NodeMakingHelpers import \
-                makeStatementExpressionOnlyReplacementNode
-
+            # Need to keep the boolean check.
             if truth_value is None:
                 condition = ExpressionBuiltinBool(
                     value      = condition,
                     source_ref = condition.getSourceReference()
                 )
 
-            # With both branches eliminated, the condition remains as a side
-            # effect.
-            result = makeStatementExpressionOnlyReplacementNode(
-                expression = condition,
-                node       = self
+            if condition.mayHaveSideEffects():
+                # With both branches eliminated, the condition remains as a side
+                # effect.
+                result = makeStatementExpressionOnlyReplacementNode(
+                    expression = condition,
+                    node       = self
+                )
+
+                return result, "new_statements", """\
+Both branches have no effect, reduced to evaluate condition."""
+            else:
+                return None, "new_statements", """\
+Removed conditional statement without effect."""
+
+        # Note: Checking the condition late, so that the surviving branch got
+        # processed already. Returning without doing that, will corrupt the SSA
+        # results. TODO: Could pretend the other branch didn't exist to save
+        # complexity the merging of processing.
+        if truth_value is not None:
+            if truth_value is True:
+                choice = "true"
+
+                new_statement = self.getBranchYes()
+            else:
+                choice = "false"
+
+                new_statement = self.getBranchNo()
+
+            new_statement = wrapStatementWithSideEffects(
+                new_node   = new_statement,
+                old_node   = condition,
+                allow_none = True # surviving branch may empty
             )
 
-            return result, "new_statements", """\
-Both branches have no effect, reduced to evaluate condition."""
+            return new_statement, "new_statements", """\
+Condition for branch was predicted to be always %s.""" % choice
 
+        # If there is no "yes" branch, remove that. Maybe a bad idea though.
         if yes_branch is None:
             # Would be eliminated already, if there wasn't any "no" branch
             # either.
@@ -436,32 +684,7 @@ Both branches have no effect, reduced to evaluate condition."""
             )
 
             return new_statement, "new_statements", """\
-Empty 'yes' branch for condition was replaced with inverted condition check."""
-
-        # Note: Checking the condition late, so that the surviving branch got
-        # processed already. Returning without doing that, will corrupt the SSA
-        # results. TODO: Could pretend the other branch didn't exist to save
-        # complexity the merging of processing.
-        if truth_value is not None:
-            from .NodeMakingHelpers import wrapStatementWithSideEffects
-
-            if truth_value is True:
-                choice = "true"
-
-                new_statement = self.getBranchYes()
-            else:
-                choice = "false"
-
-                new_statement = self.getBranchNo()
-
-            new_statement = wrapStatementWithSideEffects(
-                new_node   = new_statement,
-                old_node   = condition,
-                allow_none = True # surviving branch may empty
-            )
-
-            return new_statement, "new_statements", """\
-Condition for branch was predicted to be always %s.""" % choice
+Empty 'yes' branch for conditional statement treated with inverted condition check."""
 
         return self, None, None
 
