@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -266,6 +266,15 @@ try:
 except Exception as e:
     print("Too many args, some keywords", repr(e))
 
+try:
+    print(sum())
+except Exception as e:
+    print("Disallowed without args", repr(e))
+
+x = range(17)
+print("Sum of range(17) is", sum(x))
+print("Sum of range(17) starting with 5 is", sum(x, 5))
+
 try:
     print(str('1', offer = 2))
 except Exception as e:
