@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -17,25 +17,23 @@
 #
 import sys, os
 
-# Find common code relative in file system. Not using packages for test stuff.
+# Find nuitka package relative to us.
 sys.path.insert(
     0,
     os.path.normpath(
         os.path.join(
             os.path.dirname(os.path.abspath(__file__)),
+            "..",
             ".."
         )
     )
 )
-from test_common import (
+from nuitka.tools.testing.Common import (
     executeReferenceChecked,
-    checkReferenceCount,
-    my_print,
+    checkDebugPython
 )
 
-if not hasattr(sys, "gettotalrefcount"):
-    my_print("Warning, using non-debug Python makes this test ineffective.")
-    sys.gettotalrefcount = lambda : 0
+checkDebugPython()
 
 
 def simpleFunction1():
@@ -108,6 +106,13 @@ def simpleFunction4():
         pass
 
 
+def simpleFunction5():
+    def g():
+        yield from (2,3)
+
+    return list( g() )
+
+
 
 # These need stderr to be wrapped.
 tests_stderr = (3, 4)
