@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -19,28 +19,31 @@
 #     limitations under the License.
 #
 
-import os, sys, shutil
+import os
+import sys
 
-# Find common code relative in file system. Not using packages for test stuff.
+# Find nuitka package relative to us.
 sys.path.insert(
     0,
     os.path.normpath(
         os.path.join(
             os.path.dirname(os.path.abspath(__file__)),
+            "..",
             ".."
         )
     )
 )
-
-from test_common import (
+from nuitka.tools.testing.Common import (
     my_print,
     setup,
     hasModule,
     compareWithCPython,
     decideFilenameVersionSkip,
     getRuntimeTraceOfLoadedFiles,
-    createSearchMode
+    createSearchMode,
+    reportSkip
 )
+from nuitka.utils.FileOperations import removeDirectory
 
 python_version = setup(needs_io_encoding = True)
 
@@ -78,14 +81,11 @@ for filename in sorted(os.listdir('.')):
         # which should be considered irrelevant by now.
         if python_version.startswith("2.6") or \
            python_version.startswith("3.2"):
-            my_print("Skipping", filename, "not relevant.")
+            reportSkip("irrelevant Python version", ".", filename)
             continue
 
         if not hasModule("PySide.QtCore"):
-            my_print(
-                "Skipping", filename, "PySide not installed for",
-                python_version, "but test needs it."
-            )
+            reportSkip("PySide not installed for this Python version, but test needs it", ".", filename )
             continue
 
         # For the warnings.
@@ -96,14 +96,11 @@ for filename in sorted(os.listdir('.')):
         # which should be considered irrelevant by now.
         if python_version.startswith("2.6") or \
            python_version.startswith("3.2"):
-            my_print("Skipping", filename, "not relevant.")
+            reportSkip("irrelevant Python version", ".", filename)
             continue
 
         if not hasModule("PyQt4.QtGui"):
-            my_print(
-                "Skipping", filename, "PyQt4 not installed for",
-                python_version, "but test needs it."
-            )
+            reportSkip("PyQt4 not installed for this Python version, but test needs it", ".", filename)
             continue
 
         # For the plug-in information.
@@ -111,10 +108,7 @@ for filename in sorted(os.listdir('.')):
 
     if "Idna" in filename:
         if not hasModule("idna.core"):
-            my_print(
-                "Skipping", filename, "idna not installed for",
-                python_version, "but test needs it."
-            )
+            reportSkip("idna not installed for this Python version, but test needs it", ".", filename)
             continue
 
         # For the warnings of Python2.
@@ -126,14 +120,11 @@ for filename in sorted(os.listdir('.')):
         # which should be considered irrelevant by now.
         if python_version.startswith("2.6") or \
            python_version.startswith("3.2"):
-            my_print("Skipping", filename, "not relevant.")
+            reportSkip("irrelevant Python version", ".", filename)
             continue
 
         if not hasModule("PyQt5.QtGui"):
-            my_print(
-                "Skipping", filename, "PyQt5 not installed for",
-                python_version, "but test needs it."
-            )
+            reportSkip("PyQt5 not installed for this Python version, but test needs it", ".", filename)
             continue
 
         # For the plug-in information.
@@ -154,14 +145,11 @@ for filename in sorted(os.listdir('.')):
         # which should be considered irrelevant by now.
         if python_version.startswith("2.6") or \
            python_version.startswith("3.2"):
-            my_print("Skipping", filename, "not relevant.")
+            reportSkip("irrelevant Python version", ".", filename)
             continue
 
         if not hasModule("pygtk"):
-            my_print(
-                "Skipping", filename, "pygtk not installed for",
-                python_version, "but test needs it."
-            )
+            reportSkip("pygtk not installed for this Python version, but test needs it", ".", filename)
             continue
 
         # For the warnings.
@@ -169,39 +157,27 @@ for filename in sorted(os.listdir('.')):
 
     if filename.startswith("Win"):
         if os.name != "nt":
-            my_print("Skipping", filename, "windows only.")
+            reportSkip("Windows only test", ".", filename)
             continue
 
     if filename == "Win32ComUsing.py":
         if not hasModule("win32com"):
-            my_print(
-                "Skipping", filename, "win32com not installed for",
-                python_version, "but test needs it."
-            )
+            reportSkip("win32com not installed for this Python version, but test needs it", ".", filename)
             continue
 
     if filename == "LxmlUsing.py":
         if not hasModule("lxml.etree"):
-            my_print(
-                "Skipping", filename, "lxml.etree not installed for",
-                python_version, "but test needs it."
-            )
+            reportSkip("lxml.etree not installed for this Python version, but test needs it", ".", filename)
             continue
 
     if filename == "TkInterUsing.py":
         if python_version.startswith("2"):
             if not hasModule("Tkinter"):
-                my_print(
-                    "Skipping", filename, "Tkinter not installed for",
-                    python_version, "but test needs it."
-                )
+                reportSkip("Tkinter not installed for this Python version, but test needs it", ".", filename)
                 continue
         else:
             if not hasModule("tkinter"):
-                my_print(
-                    "Skipping", filename, "tkinter not installed for",
-                    python_version, "but test needs it."
-                )
+                reportSkip("tkinter not installed for this Python version, but test needs it", ".", filename)
                 continue
 
             # For the warnings.
@@ -210,43 +186,30 @@ for filename in sorted(os.listdir('.')):
 
     if filename == "FlaskUsing.py":
         if not hasModule("flask"):
-            my_print(
-                "Skipping", filename, "flask not installed for",
-                python_version, "but test needs it."
-            )
+            reportSkip( "flask not installed for this Python version, but test needs it", ".", filename)
             continue
 
         # For the warnings.
         extra_flags.append("ignore_stderr")
 
     if filename == "NumpyUsing.py":
+        # TODO: Disabled for now.
+        reportSkip("numpy.test not fully working yet", ".", filename)
+        continue
+
         if not hasModule("numpy"):
-            my_print(
-                "Skipping", filename, "numpy not installed for",
-                python_version, "but test needs it."
-            )
+            reportSkip("numpy not installed for this Python version, but test needs it", ".", filename)
             continue
 
         extra_flags.append("plugin_enable:data-files")
 
     if filename == "PmwUsing.py":
         if not hasModule("Pwm"):
-            my_print(
-                "Skipping", filename, "Pwm not installed for",
-                python_version, "but test needs it."
-            )
+            reportSkip("Pwm not installed for this Python version, but test needs it", ".", filename)
             continue
 
         extra_flags.append("plugin_enable:pmw-freeze")
 
-    if filename not in ("PySideUsing.py", "PyQt4Using.py", "PyQt5Using.py",
-                        "PyQt4Plugins.py", "PyQt5Plugins.py", "GtkUsing.py",
-                        "LxmlUsing.py", "Win32ComUsing.py", "IdnaUsing.py",
-                        "NumpyUsing.py", "FlaskUsing.py"):
-        extra_flags += [
-            "no_site"
-        ]
-
     my_print("Consider output of recursively compiled program:", filename)
 
     # First compare so we know the program behaves identical.
@@ -274,6 +237,7 @@ for filename in sorted(os.listdir('.')):
     for loaded_filename in loaded_filenames:
         loaded_filename = os.path.normpath(loaded_filename)
         loaded_filename = os.path.normcase(loaded_filename)
+        loaded_basename = os.path.basename(loaded_filename)
 
         if loaded_filename.startswith(current_dir):
             continue
@@ -293,6 +257,9 @@ for filename in sorted(os.listdir('.')):
         if loaded_filename.startswith("/tmp/"):
             continue
 
+        if loaded_filename.startswith("/run/"):
+            continue
+
         if loaded_filename.startswith("/usr/lib/locale/"):
             continue
 
@@ -302,37 +269,35 @@ for filename in sorted(os.listdir('.')):
         if loaded_filename.startswith("/usr/share/X11/locale/"):
             continue
 
-        if loaded_filename.startswith("/lib/libc.") or \
-           loaded_filename.startswith("/lib64/libc."):
+        # Themes may of course be loaded.
+        if loaded_filename.startswith("/usr/share/themes"):
+            continue
+        if "gtk" in loaded_filename and "/engines/" in loaded_filename:
             continue
 
-        if loaded_filename.startswith("/lib/libdl.") or \
-           loaded_filename.startswith("/lib64/libdl."):
+        # Taking these from system is harmless and desirable
+        if loaded_basename.startswith((
+            "libz.so",
+            "libutil.so",
+            "libgcc_s.so",
+            "libm.so.",
+        )):
             continue
 
-        if loaded_filename.startswith("/lib/libm.") or \
-           loaded_filename.startswith("/lib64/libm."):
-            continue
-
-        if loaded_filename.startswith("/lib/libz.") or \
-           loaded_filename.startswith("/lib64/libz."):
-            continue
-
-        if loaded_filename.startswith("/lib/libutil.") or \
-           loaded_filename.startswith("/lib64/libutil."):
-            continue
-
-        if loaded_filename.startswith("/lib/libpthread.") or \
-           loaded_filename.startswith("/lib64/libpthread."):
-            continue
-
-        if loaded_filename.startswith("/lib/libgcc_s.") or \
-           loaded_filename.startswith("/lib64/libgcc_s."):
+        # System C libraries are to be expected.
+        if loaded_basename.startswith((
+            "libc.so.",
+            "libpthread.so.",
+            "libdl.so.",
+            "libm.so.",
+        )):
             continue
 
         # Loaded by C library potentially for DNS lookups.
-        if os.path.basename(loaded_filename).startswith("libnss_") or \
-           os.path.basename(loaded_filename).startswith("libnsl"):
+        if loaded_basename.startswith((
+            "libnss_",
+            "libnsl",
+        )):
             continue
 
         # Loaded by dtruss on MacOS X.
@@ -340,7 +305,7 @@ for filename in sorted(os.listdir('.')):
             continue
 
         # Loaded by cowbuilder and pbuilder on Debian
-        if os.path.basename(loaded_filename) == ".ilist":
+        if loaded_basename == ".ilist":
             continue
         if "cowdancer" in loaded_filename:
             continue
@@ -354,14 +319,18 @@ for filename in sorted(os.listdir('.')):
            loaded_filename in ("/home", "/data", "/root"):
             continue
 
-        if os.path.basename(loaded_filename) == "gconv-modules.cache":
+        # For Debian builders, /build is OK too.
+        if loaded_filename.startswith("/build/") or \
+           loaded_filename == "/build":
             continue
 
         # TODO: Unclear, loading gconv from filesystem of installed system
         # may be OK or not. I think it should be.
+        if loaded_basename == "gconv-modules.cache":
+            continue
         if "/gconv/" in loaded_filename:
             continue
-        if os.path.basename(loaded_filename).startswith("libicu"):
+        if loaded_basename.startswith("libicu"):
             continue
 
         # Loading from caches is OK.
@@ -392,6 +361,15 @@ for filename in sorted(os.listdir('.')):
               "/dist-packages/gobject":
             continue
 
+        # PyQt5 seems to do this, but won't use contents then.
+        if loaded_filename in (
+            "/usr/lib/x86_64-linux-gnu/qt5/plugins",
+            "/usr/lib/x86_64-linux-gnu/qt5",
+            "/usr/lib/x86_64-linux-gnu",
+            "/usr/lib"
+        ):
+            continue
+
         if loaded_filename == "/usr/bin/python3.2mu":
             continue
 
@@ -408,12 +386,11 @@ for filename in sorted(os.listdir('.')):
             continue
 
         # Looking at site-package dir alone is alone.
-        if loaded_filename.endswith("site-packages"):
+        if loaded_filename.endswith(("site-packages", "dist-packages")):
             continue
 
-        loaded_basename = os.path.basename(loaded_filename).upper()
         # Windows baseline DLLs
-        if loaded_basename in (
+        if loaded_basename.upper() in (
             "SHELL32.DLL","USER32.DLL","KERNEL32.DLL",
             "NTDLL.DLL", "NETUTILS.DLL", "LOGONCLI.DLL", "GDI32.DLL",
             "RPCRT4.DLL", "ADVAPI32.DLL", "SSPICLI.DLL", "SECUR32.DLL",
@@ -462,16 +439,21 @@ for filename in sorted(os.listdir('.')):
             "FIREWALLAPI.DLL", "FLTLIB.DLL", "MRMCORER.DLL", "NTASN1.DLL",
             "SECHOST.DLL", "SETTINGSYNCPOLICY.DLL", "SHCORE.DLL", "TBS.DLL",
             "TWINAPI.APPCORE.DLL", "TWINAPI.DLL", "VIRTDISK.DLL",
-            "WEBSOCKET.DLL", "WEVTAPI.DLL", "WINMMBASE.DLL", "WMICLNT.DLL"):
+            "WEBSOCKET.DLL", "WEVTAPI.DLL", "WINMMBASE.DLL", "WMICLNT.DLL",
+            "UCRTBASE.DLL"):
             continue
 
         # Win API can be assumed.
-        if loaded_basename.startswith("API-MS-WIN"):
+        if loaded_basename.upper().startswith("API-MS-WIN"):
             continue
 
         # MSVC run time DLLs, seem to sometimes come from system. TODO:
         # clarify if that means we did it wrong.
-        if loaded_basename in ("MSVCRT.DLL", "MSVCR90.DLL"):
+        if loaded_basename.upper() in ("MSVCRT.DLL", "MSVCR90.DLL"):
+            continue
+
+        # These stopped being loaded by system on Windows 10.
+        if loaded_basename.upper() in ("MSVCP_WIN.DLL", "WIN32U.DLL"):
             continue
 
         my_print("Should not access '%s'." % loaded_filename)
@@ -480,6 +462,6 @@ for filename in sorted(os.listdir('.')):
     if illegal_access:
         sys.exit(1)
 
-    shutil.rmtree(filename[:-3] + ".dist")
+    removeDirectory(filename[:-3] + ".dist", ignore_errors = True)
 
 search_mode.finish()
